package cn.com.qmth.stmms.biz.exam.service.impl;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.bean.ScoreVerifyVo;
import cn.com.qmth.stmms.biz.exam.dao.ScoreVerifyDao;
import cn.com.qmth.stmms.biz.exam.enums.ScoreVerifyRule;
import cn.com.qmth.stmms.biz.exam.model.ScoreVerify;
import cn.com.qmth.stmms.biz.exam.query.ScoreVerifyQuery;
import cn.com.qmth.stmms.biz.exam.service.ScoreVerifyService;
import cn.com.qmth.stmms.biz.exception.StatusException;

@Service
public class ScoreVerifyServiceImpl extends BaseQueryService<ScoreVerify> implements ScoreVerifyService {
	
	@PersistenceContext
    private EntityManager entityManager;
	
	@Autowired
	private ScoreVerifyDao scoreVerifyDao;

	@Override
	public List<ScoreVerifyVo> findByQuery(ScoreVerifyQuery req) {
		int offset=(req.getPageNumber() - 1) * req.getPageSize();
		StringBuilder sql = new StringBuilder();
		sql.append("select t.id studentId,t.student_code studentCode,t.name studentName,t.subject_code subjectCode,t.subject_name subjectName, ");
		sql.append("t.is_upload upload,t.subjective_score subjectiveScore,t.subjective_score_list subjectiveScoreList,t.objective_score objectiveScore, ");
		sql.append("u.name scoreVerifyUserName,t.score_check_time scoreVerifyTime ");
		sql.append("from eb_exam_student t left join b_user u on t.score_check_user=u.id where t.exam_id="+req.getExamId());
		sql.append(getWhereSql(req));
		sql.append("order by t.student_code,t.id ");
        sql.append("limit "+offset+","+req.getPageSize());
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(ScoreVerifyVo.class));
        return query.getResultList();
	}
	
	private String getWhereSql(ScoreVerifyQuery req) {
		StringBuilder sql = new StringBuilder();
		if(StringUtils.isNotBlank(req.getParam().getStudentCode())) {
			sql.append(" and t.student_code='"+req.getParam().getStudentCode()+"'");
		}
		if(StringUtils.isNotBlank(req.getParam().getStudentName())) {
			sql.append(" and t.name='"+req.getParam().getStudentName()+"'");
		}
		ScoreVerifyRule rule=req.getParam().getRule();
		if(rule!=null) {
			if(ScoreVerifyRule.RULE1.equals(rule)) {
				
			}else if(ScoreVerifyRule.RULE2.equals(rule)) {
				
			}else if(ScoreVerifyRule.RULE3.equals(rule)) {
				
			}
		}
		return sql.toString();
	}


	@Override
	public Integer countByQuery(ScoreVerifyQuery req) {
		StringBuilder sql = new StringBuilder();
		sql.append("select count(1) ");
		sql.append("from eb_exam_student t where t.exam_id="+req.getExamId());
		sql.append(getWhereSql(req));
        Query query = entityManager.createNativeQuery(sql.toString());
        return Integer.valueOf(query.getResultList().get(0).toString());
	}

	@Override
	public ScoreVerify findByExamIdAndUserId(Integer examId, Integer id) {
		return scoreVerifyDao.findByExamIdAndUserId(examId, id);
	}

	@Transactional
	@Override
	public void createQuery(ScoreVerifyQuery query) {
		if (query.getParam() == null || (StringUtils.isBlank(query.getParam().getStudentCode())
				&& StringUtils.isBlank(query.getParam().getStudentName()) && query.getParam().getRule() == null)) {
			throw new StatusException("请选择至少一个条件");
		}
		if (ScoreVerifyRule.RULE2.equals(query.getParam().getRule())
				|| ScoreVerifyRule.RULE3.equals(query.getParam().getRule())) {
			if(query.getParam().getRuleCompare()==null) {
				throw new StatusException("请选择分数匹配条件");
			}
			if(query.getParam().getCompareScore()==null) {
				throw new StatusException("请填写匹配分值");
			}
		}
		ScoreVerify sv=new ScoreVerify();
		sv.setCreateTime(new Date());
		sv.setExamId(query.getExamId());
		sv.setUserId(query.getUserId());
		Gson gson = new Gson();
		sv.setParams(gson.toJson(query.getParam()));
		scoreVerifyDao.save(sv);
	}

}
